//
//  Controller.h
//  StreamSDKConLib
//
//  Copyright (c) 2014 StreamUnlimited. All rights reserved.
//

#ifndef __StreamSDKConLib__Controller__
#define __StreamSDKConLib__Controller__

#include <Poco/NotificationCenter.h>
#include <vector>
#include "Screen.h"
#include "EventPoller.h"
#include "Commands.h"

namespace StreamUnlimited { namespace StreamAPI {
    enum GroupingCommand {
       eUngroup = 0,
       eGroup = 1
    };

    // this is the interface for remote browsing application.
    // it must be implemented for every single remote access protocol (StreamAPI/StreamAPI)
    class Controller : public RemoteAPI::RemoteBrowser {
        private:
            Poco::NotificationCenter  _notificationCenter;
            Commands                _commands;
            EventPoller             _eventPoller;
            RemoteAPI::ConnectionState _connected;
            std::string             _currentHost;
            int                     _currentPort;
            RemoteAPI::ViewType     _viewType;
            RowsRolesVector         _currentContextMenuItemsCache;
            int                     _currentContextMenuViewItemsTotalCount;
            ScreenStack             _contextMenuStack;
            RowsRolesVector         _currentViewItemsCache;
            int                     _currentViewItemsTotalCount;
            std::vector<std::string>_currentViewSearchTitles;
            std::vector<std::string>_currentViewSearchPathes;
            ScreenStack             _screenStack;
            int                     _processingCnt;
            RemoteAPI::PlayStatus   _playStatus;
            RemoteAPI::VolumeStatus _volumeStatus;
            bool                    _mute;
            RemoteAPI::StandbyState _standby;
            bool                    _amazonLoggedIn;

            Poco::Mutex             _cs;
            Poco::Mutex             _contextCS;


            RemoteAPI::Message  _currentMessage;
            RemoteAPI::Alert    _currentAlert;

            std::string             _anchor;

        private:
            void clearCurrentViewItemsCache();
            void clearCurrentContextMenuViewItemsCache();

            // notifications from lower levels

            // as a requestResponse somethimes is possible to get redirect and close paths commands
            void     onRedirectNotification(Redirect *pNf);

            void  onContainerContentChanged(ContainerContentChanged *pNf);
            void          onPlayTimeChanged(PlayTimeChanged *pNf);
            void            onVolumeChanged(VolumeChanged *pNf);
            void     onNotificationsChanged(NotificationsChanged *pNf);
            void              onMuteChanged(MuteChanged *pNf);
            void    onAmazonLoggedInChanged(AmazonLoggedInChanged *pNf);
            void           onStandbyChanged(StandbyChanged *pNf);
            void    onPlayerPlayModeChanged(PlayerPlayModeChanged *pNf);
            void  onPlayerPlayStatusChanged(PlayerPlayStatusChanged *pNf);

            void onPlayerError(PlayerError *pNf);
            void onBrowsingError(BrowsingError *pNf);
            void onNetworkError(NetworkError *pNf);
            void onEventQueueError(EventQueueError *pNf);

            bool reconnect();
            RemoteAPI::ViewType getCurrentViewType();

            void initializeEventHandlers();
            void closeEventHandlers();
            bool cacheItemIfNecessary(const uint &position);
            RemoteAPI::PlayState playStateConvert(const PlayState &state);

            bool closePath(const std::string &path);
            void setProcessing(const bool &processing);
            bool setItem(RowsRolesVector &cache, const int &position, const std::string &editData);

            bool getViewFromPosition(const int &position);
            RolesValuesMap getRolesMapFromPosition(int position);

            void setAnchor(const std::string &path);
            std::string getAnchor();
            bool setScreenToAnchor();

            int getCurrentPlaylistId();

            void closeContext(std::string path);

        public:
            explicit Controller(RemoteAPI::RemoteBrowserObserver* const observer);
            virtual ~Controller();

            virtual std::string getFriendlyName(const std::string &address, int port);

            virtual std::string setFriendlyName(const std::string &address, int port, const std::string &name);

            virtual bool ping(const std::string &address, int port);

            // connect to the remote server
            virtual bool                connect(const std::string &address, const int &port);

            // disconnect from the remote server
            virtual bool             disconnect(const bool &force, const bool &reset);

            // ask for full UI refresh
            virtual bool refresh();

            // cancel current network operation
            virtual bool                 cancel();

            // returns the current connection state (connected/disconnected)
            virtual const RemoteAPI::ConnectionState &getConnectionState();

            //
            // Browsing. All API calls are blocking!
            //

            // get the number of items in current browsing view
            virtual int                getNumItems();
            virtual void       getNumItemsAndDepth(int &numItemsValue, int &depth);
            virtual std::vector<std::string> getSearchTitles();
            virtual void search(std::string searchString, int option);

            virtual RemoteAPI::VolumeStatus getVolume();
            // cancel current browse operation
            virtual bool           browseCancel();

            // get the processing flag
            virtual bool           isProcessing();

            virtual bool         addToFavorites();
            virtual bool    removeFromFavorites(const int &row);

            // get depth of our view
            virtual int            getCurrentDepth();
            virtual int getCurrentContextMenuDepth();

            // fetch items from the current view
            virtual std::vector<RemoteAPI::RowEntry> getItems(const int &fromPosition, const int &numItems);
            virtual std::vector<RemoteAPI::RowEntry> getContextMenuItems(const int &fromPosition, const int &numItems);

            //! Fetch items from the play queue
             //virtual std::vector<RowEntry> getPlayQueueItems(const int &fromPosition, const int &numItems);

            // set items to the play queue (for play queue only!)
            //virtual bool setPlayQueueItems(const SetRowsPlayMode &mode, const int &queueIndex, const std::vector<RowEntry> &rows);

            // remove items from the play queue
            //virtual bool removePlayQueueItems(const int &startRow, const int &count);

            // play item at given position
            virtual bool                playItem(const int &position/*, const bool &fromQueue*/);
            // Play item at given postion in context menu
            virtual bool     playContextMenuItem(const int &postion);

            // browse into item at given position
            virtual bool                browseItem(const int &position);

            // invoke query item at given position with queryStr set
            virtual bool                invokeQuery(const int &position, const std::string queryStr);

            // browse into the context menu of the item at the current position
            virtual bool getContextMenu(RemoteAPI::ContextMenu &contextMenu);

            virtual bool closeContextMenu();

            // browse into context menu item at given position
            virtual bool                browseContextMenuItem(const int &position);

            virtual bool                browseIntoContextMenu(const int &position);

            virtual bool                browseIntoContextMenu(const std::string &path);

            // return name of the current view
            virtual     std::string getCurrentTitle();

            // return name of the parent view
            virtual     std::string getParentTitle();

            // request view type
            virtual RemoteAPI::ViewType getViewType();

            virtual bool changeViewType(const RemoteAPI::ViewType &viewType);

            // jump to the parent view
            bool                      browseParent();
            bool                      browseContextMenuParent();

            // jump to the top view
            virtual bool                goHome();

            virtual bool               setItem(const int &position, const std::string &editData);
            virtual bool    setContextMenuItem(const int &position, const std::string &editData);

            //
            // Playback control
            //
            virtual bool        pause();
            virtual bool         stop();
            virtual bool         next();
            virtual bool     previous();
                virtual bool         like();
            virtual bool      dislike();
            virtual bool   fastRewind();
            virtual bool  fastForward();
            virtual bool     stopScan();

            // Unit: seconds
            virtual bool setSeek2Time(const int &seconds);

            virtual bool    setRepeat(const RemoteAPI::RepeatMode &repeat);
            virtual bool   setShuffle(const RemoteAPI::RandomMode &random);
            virtual bool      setMute(const bool &mute);
            virtual bool    setVolume(const int &volume);
            virtual bool setStandbyState(const RemoteAPI::StandbyState state);

            virtual bool alexaVoiceServicePresent();
#ifdef USE_NSDK_AMAZON_LOGIN
            virtual bool amazonCompanionUrl(std::string &url);
            virtual bool amazonLoginBlob(std::string &blob);
            virtual bool amazonSetTokenBlob(const std::string &blob);
#else
            virtual bool amazonLoginDeviceInfo(RemoteAPI::AmazonLoginDeviceInfo &info);
            virtual bool amazonLoginAppCode(const RemoteAPI::AmazonLoginAppCode &appCode);
#endif
            virtual bool amazonLoggedIn();
            virtual bool amazonLogout();

            // retrieve current message
            virtual bool   getMessage(RemoteAPI::Message &message);
            virtual bool confirmMessage(const RemoteAPI::Message &message);
            virtual bool  cancelMessage(const RemoteAPI::Message &message);

            // retrieve current alert
            virtual bool   getAlert(RemoteAPI::Alert &alert);
            virtual bool closeAlert(const RemoteAPI::Alert &alert);

            virtual bool isGroupingSupported(const std::string &ipAddress, int port);
            virtual bool group(const std::string& ipAddress, int port, const std::string& sourceUUID, const std::string& sourceIpAddress, const std::string& sourceName);
            virtual bool ungroup(const std::string& ipAddress, int port);
            virtual bool persGroups(const std::string& ipAddress, int port, const std::string& groupName, std::vector<std::string>& receiversUUID);
            virtual std::string getGroup(const std::string& uuid, const std::string &ipAddress, int port);
            virtual std::vector<std::string> getSlaves(const std::string &ipAddress, int port);

            //
            // Direct access API (no iteration necessary)
            //
            virtual bool browse(const std::string &path, const std::string &title, const int &startPos, const int &count, const bool direct = true);
            virtual std::string get(const std::string &path);
            virtual bool activate(const std::string &path, const std::string &data = "");
            virtual bool activate(const std::string &path, const std::string &type, const std::string &value);
            virtual bool activateItem(int position, const std::string &data = "");
            virtual bool activateItem(int position, const std::string &type, const std::string &value);
            virtual bool set(const std::string &path, const std::string& value);

            // Browsing with anchor
            virtual bool browseAndSetAnchor(const std::string &path, const int &startPos, const int &count);
            virtual bool browseItemAndSetAnchor(const int &position);
            virtual bool browseAnchor();
            virtual bool browseFromAnchor(const std::string &path, const int &startPos, const int &count);

            // Playlist Control
            virtual bool browsePlaylists();
            virtual bool browsePlayqueue();

            virtual bool createPlaylist(const std::string &name);
            virtual bool deletePlaylist(int id);

            virtual bool clearPlaylist(int id);
            virtual bool setPlaylistName(int id, const std::string &name);
            virtual bool setPlaylistShuffle(int id, RemoteAPI::RandomMode shuffle);
            virtual bool setPlaylistRepeat(int id, RemoteAPI::RepeatMode repeat);

            virtual bool movePlaylistItems(std::vector<int> indicies, int to);
            virtual bool addPlaylistItems(int id, RemoteAPI::AddItemMode addMode, std::vector<std::string> itemPaths);
            virtual bool removePlaylistItems(int id, std::vector<int> indicies);

            // Collection Control
            virtual bool createCollection(const std::string &name);
            virtual bool deleteCollection(int id);
            virtual bool setCollectionName(int id, const std::string &name);
            virtual bool addPlaylistToCollection(int collectionId, int playlistId);
            virtual bool removePlaylistFromCollection(int collectionId, int playlistId);
    };
} }

#endif /* defined(__StreamSDKConLib__Controller__) */
