//
//  Notifications.h
//  StreamSDKConLib
//
//  Copyright (c) 2014 StreamUnlimited. All rights reserved.
//

#ifndef __StreamSDKConLib__Notifications__
#define __StreamSDKConLib__Notifications__

#include <string>
#include <vector>
#include <Poco/Notification.h>
#include "RemoteBrowser.h"

namespace StreamUnlimited { namespace StreamAPI {
    enum PlayState {
        ePlayStateStop,
        ePlayStatePlay,
        ePlayStatePause,
        ePlayStateTransition,
        ePlayStateScanInc,
        ePlayStateScanDec,
        ePlayStateTrackStopped,
        ePlayStateUndefined
    };

    class BaseNotification: public Poco::Notification {
    };

    class Redirect : public BaseNotification {
    public:
        std::string redirectPath;
        std::vector<std::string> pathsToClose;
        bool closeContextMenus;

        Redirect()
        {
            clear();
        }

        void clear()
        {
            redirectPath.clear();
            pathsToClose.clear();
            closeContextMenus = false;
        }

        bool isEmpty()
        {
             if (!redirectPath.empty())
                 return false;

             if (!pathsToClose.empty())
                 return false;

             if (pathsToClose.size() > 0)
                 return false;

             if (closeContextMenus)
                 return false;

            return true;
        }
    };

    class ViewTypeChanged : public BaseNotification {
    };

    class ContainerContentChanged : public BaseNotification {
    public:
        std::string _path;
        explicit ContainerContentChanged(std::string path) {_path = path;}
    };

    class PlayTimeChanged : public BaseNotification {
    public:
        Poco::Int64 _time;
        explicit PlayTimeChanged(Poco::Int64 time){_time = time;}
    };

    class VolumeChanged : public BaseNotification {
    public:
        //! Constructor
        VolumeChanged()
        {
            clear();
        }

        void clear()
        {
            _currentVolume = 0;
            _minVolume     = 0;
            _maxVolume     = 100;
            _volumeStep    = 1;
        }

        int     _currentVolume;
        int     _minVolume;
        int     _maxVolume;
        int     _volumeStep;
    };

    class StandbyChanged : public BaseNotification {
    public:
        int _standby;
        explicit StandbyChanged(int standby){_standby = standby;}
    };

    class MuteChanged : public BaseNotification {
    public:
        bool _mute;
        explicit MuteChanged(bool mute){_mute = mute;}
    };

    class AmazonLoggedInChanged : public BaseNotification {
    public:
        bool _amazonLoggedIn;
        explicit AmazonLoggedInChanged(bool amazonLoggedIn){_amazonLoggedIn = amazonLoggedIn;}
    };

    class PlayStatusChanged : public BaseNotification {
    };

    class PlayerPlayModeChanged : public BaseNotification {
    public:
        bool _shuffle;
        int _repeat;
        explicit PlayerPlayModeChanged(int repeat, bool shuffle){_repeat=repeat; _shuffle=shuffle;}
    };

    class MqaInfo
    {
    public:
        int         _outputSampleRate;
        int         _originalSampleRate;
        bool        _customRateUsed;
        std::string _originalSampleRateCustom;
        int         _creatorId;
        std::string _state;
        std::string _fullStringMqaInfo;

        void clear()
        {
            _outputSampleRate = 0;
            _originalSampleRate = 0;
            _customRateUsed = false;
            _originalSampleRateCustom = "";
            _creatorId = 0;
            _state = "";
            _fullStringMqaInfo = "";
        }

        MqaInfo()
        {
            clear();
        }
    };

    class PlayerPlayStatusChanged : public BaseNotification {
    public:
        std::string  _source;
        std::string  _serviceName;
        std::string  _title;
        RemoteAPI::MimeType _mimeType;
        int          _repeat;
        bool         _random;
        PlayState    _playState;
        unsigned int _availableControls;
        int          _queueIndex;
        int          _queueLength;

        std::string  _metaType;
        std::string  _metaAlbumartUrl;
        std::string  _metaArtist;
        std::string  _metaAlbum;
        std::string  _metaGenre;
        std::string  _contextPath;
        bool         _liveStream;

        int          _playTime;
        int          _trackTime;
        int          _samplerate;
        int          _bitrate;
        int          _channels;
        int          _bps;
        std::string  _codec;
        MqaInfo      _mqaInfo;

        void clear()
        {
            _source = "";
            _serviceName = "";
            _title = "";
            _mimeType = RemoteAPI::eMimeType_None;
            _repeat = 0;
            _random = false;
            //_playState
            _availableControls = 0;
            _metaType = "";
            _metaAlbumartUrl = "";
            _metaArtist = "";
            _metaGenre = "";
            _contextPath = "";
            _liveStream = false;
            _playTime = 0;
            _queueIndex = -1;
            _queueLength= 0;
            _trackTime = 0;
            _samplerate = 0;
            _bitrate = 0;
            _channels = 0;
            _bps = 0;
            _codec = "";
        }

        PlayerPlayStatusChanged()
        {
            clear();
        }
    };

    class NotificationsChanged : public BaseNotification {
    public:
        std::string _path;
        std::string _caption;
        std::vector<std::string> _text;

        void clear()
        {
            _path = "";
            _caption = "";
            _text.clear();
        }

        NotificationsChanged()
        {
            clear();
        }
    };

    class PlayerError : public BaseNotification {
    public:
        std::string _id;
        std::string _caption;
        std::string _text;

        void clear()
        {
            _id = "";
            _caption = "";
            _text = "";
        }

        PlayerError()
        {
            clear();
        }
    };

    class BrowsingError : public BaseNotification {
        public:
        std::string _message;
        std::string _name;
        std::string _title;

        void clear()
        {
            _message = "";
            _name = "";
            _title = "";
        }

        BrowsingError()
        {
            clear();
        }

        BrowsingError(std::string message, std::string name,  std::string title = "")
        {
            _message = message; _name = name; _title = title;
        }
    };

    class NetworkError : public BaseNotification {
    public:
        std::string _host;
        int _port;

        NetworkError(std::string host, int port)
        {
            _host = host;
            _port = port;
        }

    };

    class EventQueueError : public BaseNotification {
    public:
        EventQueueError(){}
    };
} }

#endif /* defined(__StreamSDKConLib__Notifications__) */
